@@ -3,7 +3,8 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2018, 2019, 2020, 2021  Martin Lambers <marlam@marlam.de>
+ * Copyright (C) 2018, 2019, 2020, 2021, 2022
+ * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -34,6 +35,7 @@
 #include <arpa/inet.h>
 #include <netinet/in.h>
 #include <sys/socket.h>
+#include <netdb.h>
 #include <sys/wait.h>
 #include <time.h>
 #include <syslog.h>
@@ -44,6 +46,8 @@ extern int optind;
 
 #include "base64.h"
 #include "password.h"
+#include "eval.h"
+#include "tools.h"
 #include "xalloc.h"
 
 
@@ -249,11 +253,48 @@ int smtp_pipe(FILE* in, FILE* pipe, char* buf, size_t bufsize)
         if (!line_continues && fputc('\n', pipe) == EOF)
             return 1;
     }
-    if (fflush(pipe) != 0)
+    if (fflush(pipe) != 0 || ferror(pipe))
         return 1;
     return 0;
 }
 
+/* Get the local and remote IP addresses and host names */
+
+typedef struct {
+    char local_ip[INET6_ADDRSTRLEN+1];
+    char local_name[NI_MAXHOST];
+    char remote_ip[INET6_ADDRSTRLEN+1];
+    char remote_name[NI_MAXHOST];
+} connection_info_t;
+
+void get_connection_info(int socket, connection_info_t* info)
+{
+    struct sockaddr_storage ss;
+    socklen_t ss_len;
+    ss_len = sizeof(ss);
+    info->local_ip[0] = '\0';
+    info->local_name[0] = '\0';
+    if (getsockname(socket, (struct sockaddr*)&ss, &ss_len) == 0) {
+        if (ss.ss_family == AF_INET6) {
+            inet_ntop(AF_INET6, &(((struct sockaddr_in6*)&ss)->sin6_addr), info->local_ip, sizeof(info->local_ip));
+        } else if (ss.ss_family == AF_INET) {
+            inet_ntop(AF_INET, &(((struct sockaddr_in*)&ss)->sin_addr), info->local_ip, sizeof(info->local_ip));
+        }
+        getnameinfo((struct sockaddr*)&ss, sizeof(ss), info->local_name, sizeof(info->local_name), NULL, 0, NI_NAMEREQD);
+    }
+    ss_len = sizeof(ss);
+    info->remote_ip[0] = '\0';
+    info->remote_name[0] = '\0';
+    if (getpeername(socket, (struct sockaddr*)&ss, &ss_len) == 0) {
+        if (ss.ss_family == AF_INET6) {
+            inet_ntop(AF_INET6, &(((struct sockaddr_in6*)&ss)->sin6_addr), info->remote_ip, sizeof(info->remote_ip));
+        } else if (ss.ss_family == AF_INET) {
+            inet_ntop(AF_INET, &(((struct sockaddr_in*)&ss)->sin_addr), info->remote_ip, sizeof(info->remote_ip));
+        }
+        getnameinfo((struct sockaddr*)&ss, sizeof(ss), info->remote_name, sizeof(info->remote_name), NULL, 0, NI_NAMEREQD);
+    }
+}
+
 /* SMTP session with input and output from FILE descriptors.
  * Mails are piped to the given command, where the first occurrence of %F
  * will be replaced with the envelope-from address, and all recipient addresses
@@ -264,12 +305,15 @@ int smtp_pipe(FILE* in, FILE* pipe, char* buf, size_t bufsize)
  * 1 if the session had errors and/or was aborted
  * 2 same as 1 and the session had authentication failures
  * */
-int msmtpd_session(log_t* log, FILE* in, FILE* out,
+int msmtpd_session(log_t* log,
+        FILE* in, FILE* out,
+        const connection_info_t* info,
         const char* command,
         const char* user, const char* password, int impose_auth_delay)
 {
     char buf[SMTP_BUFSIZE];
     char buf2[SMTP_BUFSIZE];
+    char ehlo_name[SMTP_BUFSIZE];
     size_t addrlen;
     char* cmd;
     char* tmpcmd;
@@ -284,6 +328,7 @@ int msmtpd_session(log_t* log, FILE* in, FILE* out,
     log_msg(log, log_info, "starting SMTP session");
     setlinebuf(out);
     fprintf(out, "220 localhost ESMTP msmtpd\r\n");
+    /* expect EHLO or HELO */
     if (read_smtp_cmd(in, buf, SMTP_BUFSIZE) != 0) {
         log_msg(log, log_error, "client did not send initial command, session aborted");
         return 1;
@@ -293,6 +338,23 @@ int msmtpd_session(log_t* log, FILE* in, FILE* out,
         log_msg(log, log_error, "client did not start with EHLO or HELO, session aborted");
         return 1;
     }
+    /* save EHLO name for the Received header, but sanitize it */
+    if (buf[5] == '\0') {
+        strcpy(ehlo_name, "unknown");
+    } else {
+        strcpy(ehlo_name, buf + 5);
+        for (i = 0; ehlo_name[i]; i++) {
+            if (!((i == 0 && ehlo_name[i] == '[') || (ehlo_name[i + 1] == '\0' && ehlo_name[i] == ']')
+                        || (ehlo_name[i] >= 'a' && ehlo_name[i] <= 'z')
+                        || (ehlo_name[i] >= 'A' && ehlo_name[i] <= 'Z')
+                        || (ehlo_name[i] >= '0' && ehlo_name[i] <= '9')
+                        || (ehlo_name[i] == '.' || ehlo_name[i] == ':' || ehlo_name[i] == '-'))) {
+                strcpy(ehlo_name, "invalid");
+                break;
+            }
+        }
+    }
+    /* send EHLO/HELO response */
     if (user && strncasecmp(buf, "EHLO ", 5) == 0) {
         fprintf(out, "250-localhost\r\n");
         fprintf(out, "250 AUTH PLAIN\r\n");
@@ -461,6 +523,18 @@ int msmtpd_session(log_t* log, FILE* in, FILE* out,
             log_msg(log, log_error, "cannot start pipe command, session aborted");
             return 1;
         }
+        /* first add a Received: header */
+        char rfc2822_timestamp[32];
+        print_time_rfc2822(time(NULL), rfc2822_timestamp);
+        fprintf(pipe, "Received: from %s (", ehlo_name);
+        if (info->remote_name[0])
+            fprintf(pipe, "%s ", info->remote_name);
+        fprintf(pipe, "[%s])\r\n\tby ", info->remote_ip);
+        if (info->local_name[0])
+            fprintf(pipe, "%s ", info->local_name);
+        fprintf(pipe, "[%s] (msmtpd) with ESMTP;\r\n", info->local_ip);
+        fprintf(pipe, "\t%s\r\n", rfc2822_timestamp);
+        /* then pipe the mail */
         fprintf(out, "354 Send data\r\n");
         if (smtp_pipe(in, pipe, buf, SMTP_BUFSIZE) != 0) {
             fprintf(out, "554 Cannot pipe mail to command\r\n");
@@ -629,7 +703,7 @@ int parse_command_line(int argc, char* argv[],
                     char* tmp_user = xstrndup(optarg, comma - optarg);
                     char* tmp_password = NULL;
                     char* errstr = NULL;
-                    if (password_eval(comma + 1, &tmp_password, &errstr) != 0) {
+                    if (eval(comma + 1, &tmp_password, &errstr) != 0) {
                         fprintf(stderr, "%s: cannot get password: %s\n", argv[0], errstr);
                         free(tmp_user);
                         free(errstr);
@@ -708,6 +782,7 @@ int main(int argc, char* argv[])
     }
 
     /* Do it */
+    connection_info_t info;
     int ret = exit_ok;
     signal(SIGPIPE, SIG_IGN); /* Do not terminate when piping fails; we want to handle that error */
     if (inetd) {
@@ -715,7 +790,9 @@ int main(int argc, char* argv[])
         log_t log;
         log_open(log_to_syslog, log_file, &log);
         int impose_auth_delay = 1; /* since we cannot keep track of auth failures in inetd mode */
-        ret = msmtpd_session(&log, stdin, stdout, command, user, password, impose_auth_delay);
+        get_connection_info(fileno(stdin), &info);
+        ret = msmtpd_session(&log, stdin, stdout, &info, command,
+                user, password, impose_auth_delay);
         ret = (ret == 0 ? 0 : 1);
         log_close(&log);
     } else {
@@ -795,15 +872,7 @@ int main(int argc, char* argv[])
                 ret = exit_not_running;
                 break;
             }
-            char client_ip_str[INET6_ADDRSTRLEN];
-            int client_port;
-            if (ipv6) {
-                inet_ntop(AF_INET6, &sa6.sin6_addr, client_ip_str, sizeof(client_ip_str));
-                client_port = ntohs(sa6.sin6_port);
-            } else {
-                inet_ntop(AF_INET, &sa4.sin_addr, client_ip_str, sizeof(client_ip_str));
-                client_port = ntohs(sa4.sin_port);
-            }
+            get_connection_info(conn_fd, &info);
             pid_t pid = fork();
             if (pid == 0) {
                 /* Child process */
@@ -819,11 +888,12 @@ int main(int argc, char* argv[])
                 signal(SIGCHLD, SIG_DFL); /* Make popen()/pclose() work again */
                 log_t log;
                 log_open(log_to_syslog, log_file, &log);
-                log_msg(&log, log_info, "connection from %s port %d, active sessions %d (max %d), auth_delay=%s",
-                        client_ip_str, client_port,
-                        active_sessions_count + 1, MAX_ACTIVE_SESSIONS, impose_auth_delay ? "yes" : "no");
+                log_msg(&log, log_info, "connection from %s, active sessions %d (max %d), auth_delay=%s",
+                        info.remote_ip, active_sessions_count + 1, MAX_ACTIVE_SESSIONS,
+                        impose_auth_delay ? "yes" : "no");
                 FILE* conn = fdopen(conn_fd, "rb+");
-                int ret = msmtpd_session(&log, conn, conn, command, user, password, impose_auth_delay);
+                int ret = msmtpd_session(&log, conn, conn, &info, command,
+                        user, password, impose_auth_delay);
                 fclose(conn);
                 log_msg(&log, log_info, "connection closed");
                 log_close(&log);
